var child_process = require('child_process');
var fs = require('fs');
var path = require('path');
var spawn = require('child_process').spawn;
var os = require('os');
var which = require('which');

var utilpath = path.dirname( which.sync('harktool5') );
utilpath = path.join(utilpath, "..", "share", "hark") + path.sep;
utilpath = utilpath.replace(/\\/g, "/");

var userhome = process.env[process.platform == "win32" ? "USERPROFILE" : "HOME"] + path.sep;
userhome = userhome.replace(/\\/g, "/");

exports.index = function(req, res) {
    req.params[0] = "index";
    exports.harktool(req, res);
};

exports.harktool = function(req, res) {
    // console.log(req);
    res.render(req.params[0], { 'utilpath': utilpath, 'userhome': userhome });
};

// tfgen
exports.tfgen = function(req, res) {
    var value = JSON.parse(req.body.json);
    console.log("mode", req.params.mode);
    console.log("body", value);
    res.status(200).send();
};

exports.download = function(req, res) {
    var filename = req.params.filename;

    // var content = fs.readFileSync();
    res.download("file/" + filename);
};

var progress = 0;

function runner(cmd, args, filename, res, done, fail) {
    var stderr = "";
    var stdout = "";
    var whole = "";
    progress = 0;

    child = spawn(cmd, args);
    console.log("runner= ", cmd, args);

    child.stdout.on("data", function(data) {
        data = data.toString();
        stdout += data + "\n";
        whole += data + "\n";

        var keyword = "progress:";
        if (whole.lastIndexOf(keyword) != -1) {
            var line = whole.slice(whole.lastIndexOf(keyword)).split("\n")[0];
            var cur = line.split(":")[1].split("/")[0];
            var total = line.split(":")[1].split("/")[1];
            cur = parseInt(cur, 10);
            total = parseInt(total, 10);
            console.log(line, cur, total);
            progress = parseInt(100.0 * cur / total) || 0;
        }
    });

    child.stderr.on("data", function(data) {
        stderr += data.toString() + "\n";
        whole += data.toString() + "\n";
        var keyword = "progress:";
        if (whole.lastIndexOf(keyword) != -1) {
            var line = whole.slice(whole.lastIndexOf(keyword)).split("\n")[0];
            var cur = line.split(":")[1].split("/")[0];
            var total = line.split(":")[1].split("/")[1];
            console.log(line, cur, total);
            cur = parseInt(cur, 10);
            total = parseInt(total, 10);
            console.log(line, cur, total);
            progress = parseInt(100.0 * cur / total) || 0;
        }
    });

    child.on("close", function(code, signal) {
        if (code === 0) {
            if (done)
                done(stdout);
            else
                res.status(200).send({ path: filename });
        } else {
            if (fail)
                fail(stderr);
            else {
                //var command = cmd + " " + args.join(" ");
                //res.status(503).send({ stderr: whole.trim(), cmd: command });
                res.status(503).send(stderr);
            }
        }
        console.log(whole);
        child = undefined;
    });

    child.unref();
}

function validImpl(type, filepath, res) {
    // check file
    var cmd, opts;
    if (type === "mic") {
        cmd = 'harktoolcli-validxml';
        opts = ['-m', '-j', filepath];
    } else if (type === "src") {
        cmd = 'harktoolcli-validxml';
        opts = ['-s', '-j', filepath];
    } else if (type === "tf") {
        cmd = 'harktoolcli-validtf';
        opts = ['-j', filepath];
    }
    runner(cmd, opts, filepath, res, function(msg) {
        res.status(200).send(msg);
    }, function(msg) {
        res.status(415).send(msg);
    });
}

/////////////////////////////////////////////
//// validxml/validtf
////
exports.valid = function(req, res) {

    var type = req.params.filetype;
    var parameters = req.body;
    var curPath = process.cwd();

    console.log("exports.valid start");
    console.log(parameters);

    validImpl(type, curPath + "/file/" + parameters.filename, res);
};

exports.upload = function(req, res) {
    var tmp_path = req.file.path;
    var curPath = process.cwd();
    var tgt_path = curPath + "/file/" + req.file.originalname;
    var type = req.params.filetype;

    console.log("exports.upload start");
    console.log(type + ', ' + tmp_path + ', ' + tgt_path);

    // rename
    fs.rename(tmp_path, tgt_path, function(err) {
        if (err) {
            res.status(500).send("Rename error: " + err);
            return;
        } else {
            //fs.unlink(tmp_path);
            validImpl(type, tgt_path, res);
        }
    });
};


/////////////////////////////////////////////
//// genconfig
////
exports.genconfig = function(req, res) {
    var parameters = req.body;
    var cmd = 'harktoolcli-genconfig';
    var curPath = process.cwd();

    console.log("exports.genconfig start ");
    console.log(parameters);

    var args = [];

    var filename = "genconfig.xml";

    //common set


    args.push(curPath + "/file/" + filename);

    args.push("-c");
    args.push(parameters.comment);

    args.push("-s");
    args.push(parameters.syncadd);

    args.push("-o");
    args.push(parameters.tspoffset);

    args.push("-p");
    args.push(parameters.peakfrom + "," + parameters.peakto);

    args.push("-n");
    args.push(parameters.fftlen);

    args.push("-r");
    args.push(parameters.samplingrate);

    args.push("-m");
    args.push(parameters.signalmax);

    args.push("-l");
    args.push(parameters.tsplength);

    args.push("-t");
    args.push(parameters.orgtsppath);

    console.log("args=", args);
    runner('harktoolcli-genconfig', args, filename, res);
};


/////////////////////////////////////////////
//// genposxml
////
exports.genposxml = function(req, res) {
    var parameters = req.body;
    var curPath = process.cwd();

    console.log("exports.genposxml start ");
    console.log(parameters);

    console.log("parameters.outfile=", parameters.outfile);

    var filename = parameters.outfile;

    var args = [];

    //common set
    args.push(parameters.template);

    args.push("--type");
    args.push(parameters.ptype);
    args.push("--output");
    args.push(curPath + "/file/" + filename);
    if (parameters.path) {
        args.push("--path");
        args.push(parameters.path);
    }
    args.push("--id");
    args.push('0');
    if (parameters.channels) {
        args.push("--channels");
        args.push(parameters.channels);
    }

    if (parameters.template == "circle") {
        args.push("--min-azimuth");
        args.push(parameters.afrom);
        args.push("--max-azimuth");
        args.push(parameters.ato);
        args.push("--interval-azimuth");
        args.push(parameters.ainterval);

        args.push("--elevation");
        args.push(parameters.elevation);

        args.push("--min-radius");
        args.push(parameters.rfrom);
        args.push("--max-radius");
        args.push(parameters.rto);
        args.push("--interval-radius");
        args.push(parameters.rinterval);
    } else if (parameters.template == "grid") {
        args.push("--min-x_axis");
        args.push(parameters.xfrom);
        args.push("--max-x_axis");
        args.push(parameters.xto);
        args.push("--interval-x_axis");
        args.push(parameters.xinterval);

        args.push("--min-y_axis");
        args.push(parameters.yfrom);
        args.push("--max-y_axis");
        args.push(parameters.yto);
        args.push("--interval-y_axis");
        args.push(parameters.yinterval);

        args.push("--min-z_axis");
        args.push(parameters.zfrom);
        args.push("--max-z_axis");
        args.push(parameters.zto);
        args.push("--interval-z_axis");
        args.push(parameters.zinterval);
    } else if (parameters.template == "cylinder") {
        args.push("--min-azimuth");
        args.push(parameters.afrom);

        args.push("--max-azimuth");
        args.push(parameters.ato);

        args.push("--interval-azimuth");
        args.push(parameters.ainterval);

        args.push("--min-height");
        args.push(parameters.hfrom);

        args.push("--max-height");
        args.push(parameters.hto);

        args.push("--interval-height");
        args.push(parameters.hinterval);

        args.push("--min-radius");
        args.push(parameters.rfrom);

        args.push("--max-radius");
        args.push(parameters.rto);

        args.push("--interval-radius");
        args.push(parameters.rinterval);
    } else if (parameters.template == "ball") {
        args.push("--min-azimuth");
        args.push(parameters.afrom);
        args.push("--max-azimuth");
        args.push(parameters.ato);
        args.push("--interval-azimuth");
        args.push(parameters.ainterval);

        args.push("--min-elevation");
        args.push(parameters.efrom);
        args.push("--max-elevation");
        args.push(parameters.eto);
        args.push("--interval-elevation");
        args.push(parameters.einterval);

        args.push("--min-radius");
        args.push(parameters.rfrom);
        args.push("--max-radius");
        args.push(parameters.rto);
        args.push("--interval-radius");
        args.push(parameters.rinterval);
    }

    console.log("args=", args);

    runner('harktoolcli-genposxml', args, filename, res);
};

/////////////////////////////////////////////
//// updposxml
////
exports.updposxml = function(req, res) {
    var parameters = req.body;
    var curPath = process.cwd();

    console.log("exports.genposxml start ");
    console.log(parameters);

    console.log("parameters.outfile=", parameters.outfile);

    var newfilename = ((parameters.ptype === "microphone")?"":"temporary_") + parameters.outfile;
    var newfilepath = curPath + "/file/" + newfilename;

    /* Processing same as harkio_XML_internal_writePosition__() */
    var xmldata = "<hark_xml version=\"1.3\">\n";
    xmldata += "    <positions type=\"" + parameters.ptype + "\" coordinate=\"cartesian\">\n";
    for(var i=0; i<parameters.positions.length; i++) {
        pos = parameters.positions[i];
        xmldata += "        <position x=\"" + pos.x + "\" y=\"" + pos.y + "\" z=\"" + pos.z + "\" id=\"" + pos.id + "\" path=\"" + pos.path + "\"/>\n";
    }
    xmldata += "    </positions>\n";
    console.log("channels =", parameters.channels);
    if (parameters.channels instanceof Array && parameters.channels.length) {
        xmldata += "    <channels use=\"";
        for (var j=0; j<parameters.channels.length; j++) {
            xmldata += parameters.channels[j] + ";";
        }
        xmldata += "\">\n";
        xmldata += "    </channels>\n";
    }
    xmldata += "</hark_xml>\n";
    console.log(xmldata);
    fs.writeFile(newfilepath, xmldata, function(err) {
        if (err) {
            res.status(500).send("updateXml error: " + err);
            return;
        } else {
            res.status(200).send({ path: newfilename });
        }
    });
};

/////////////////////////////////////////////
//// genneighbors
////
exports.genneighbors = function(req, res) {
    var parameters = req.body;
    var curPath = process.cwd();

    console.log("exports.genneighbors start ");
    console.log(parameters);

    var args = [];
    
    if (parameters.algorithm) {
        args.push("-a");
        args.push(parameters.algorithm);
    }
    if (parameters.maxnum) {
        args.push("-n");
        args.push(parameters.maxnum);
    }
    if (parameters.threshold) {
        args.push("-t");
        args.push(parameters.threshold);
    }

    var filepath = curPath + "/file/" + parameters.filename;

    args.push(filepath);
    args.push(filepath);

    console.log("args=", args);

    runner('harktoolcli-genneighbors', args, parameters.filename, res, function(msg) {
        res.status(200).send({ path: parameters.filename });
    });
};

/////////////////////////////////////////////
//// mergexmls
////
exports.mergexmls = function(req, res) {
    var parameters = req.body;
    var curPath = process.cwd();

    console.log("exports.mergexmls start ");
    console.log(parameters);

    var args = [];

    var sourcepath = curPath + "/file/" + parameters.slist;
    args.push(sourcepath);
    args.push(curPath + "/file/" + "genconfig.xml");
    args.push("-l");
    args.push("-o");
    args.push(sourcepath);

    console.log("args=", args);
    runner('harktoolcli-mergexmls', args, parameters.slist, res);
};

/////////////////////////////////////////////
//// mergesrcxmls
////
exports.mergesrcxmls = function(req, res) {
    var parameters = req.body;
    var curPath = process.cwd();

    console.log("exports.mergesrcxmls start ");
    console.log(parameters);

    var args = [];

    var sourcepath = curPath + "/file/" + parameters.filename1;
    args.push(sourcepath);
    args.push(curPath + "/file/" + parameters.filename2);
    args.push("-l");
    args.push("-o");
    args.push(sourcepath);

    console.log("args=", args);
    runner('harktoolcli-mergexmls', args, parameters.filename1, res);
};

/////////////////////////////////////////////
//// gentf
////
exports.gentf = function(req, res) {
    var parameters = req.body;
    var curPath = process.cwd();

    console.log("exports.gentf start ");
    console.log(parameters);

    var args = [];

    console.log("parameters.gtype=", parameters.gtype);

    //common set
    args.push(parameters.gtype); //calctfimp  

    args.push("--mic-list");
    args.push(curPath + "/file/" + parameters.mlist);

    if (parameters.gtype === "calctfimp") {
        args.push("--impulse-list");
    }

    if (parameters.gtype === "calctfrec" || parameters.gtype === "calctfgeo" || parameters.gtype === "calctfmouth") {
        args.push("--tsp-list");
    }

    args.push(curPath + "/file/" + parameters.slist);

    args.push("--output-type");
    args.push(parameters.tftype); //LOC or SEP or LOC_SEP

    args.push("--output-file");
    args.push(curPath + "/file/" + parameters.zippath); //zip 

    args.push("--normalize-src");
    args.push(parameters.normsrc);

    args.push("--normalize-mic");
    args.push(parameters.normmic);

    args.push("--normalize-freq");
    args.push(parameters.normfreq);

    args.push("--direct-length");
    args.push(parameters.directlength);

    if (parameters.gtype !== "calctfgeo") {
        args.push("--reverb-length");
        args.push(parameters.reverblength);
    }

    if (parameters.resetmic == 1) {
        args.push("--reset-mic");
    }

    if (parameters.gtype === "calctfmouth") {
        args.push("--end");
        args.push(parameters.mouthend);
        args.push("--mch");
        args.push(parameters.mch);
        args.push("--hop");
        args.push(parameters.hop);
        args.push("--headmargin");
        args.push(parameters.headmargin);
    }

    args.push("--logLevel");
    args.push("E");

    console.log("args=", args);
    runner('harktool5', args, parameters.zippath, res);
};

/////////////////////////////////////////////
//// genzip2AMPcoord
////
exports.genzip2AMPcoord = function(req, res) {
    var parameters = req.body;
    var cmd = 'harktoolcli-zip2coord ';
    var curPath = process.cwd();

    console.log("exports.genzip2AMPcoord start ");
    console.log(parameters);

    var args = [];

    var filename = "genzip2AMPcoord.txt";

    //Type of Transfer function    (-t loc)    [string]    
    args.push("-t");
    args.push(parameters.tftype);

    //z_axis
    args.push("-z");
    args.push("amp");

    //input file    
    args.push(curPath + "/file/" + parameters.zippath);

    //output file   
    args.push(curPath + "/file/" + filename);

    console.log('harktoolcli-zip2coord' + args.join(" "));
    runner('harktoolcli-zip2coord', args, filename, res);
};

/////////////////////////////////////////////
//// genzip2DBcoord
////
exports.genzip2DBcoord = function(req, res) {
    var parameters = req.body;
    var cmd = 'harktoolcli-zip2coord ';
    var curPath = process.cwd();

    console.log("exports.genzip2DBcoord start ");
    console.log(parameters);

    var args = [];

    var filename = "genzip2DBcoord.txt";

    //Type of Transfer function    (-t loc)    [string]    
    args.push("-t");
    args.push(parameters.tftype);

    //z_axis
    args.push("-z");
    args.push("db");

    //input file    
    args.push(curPath + "/file/" + parameters.zippath);

    //output file   
    args.push(curPath + "/file/" + filename);

    console.log("args=", args);

    runner('harktoolcli-zip2coord', args, filename, res);
};

/////////////////////////////////////////////
//// genzip2PAHSEcoord
////
exports.genzip2PHASEcoord = function(req, res) {
    var parameters = req.body;
    var cmd = 'harktoolcli-zip2coord ';
    var curPath = process.cwd();

    console.log("exports.genzip2PHASEcoord start ");
    console.log(parameters);

    var args = [];

    var filename = "genzip2PHASEcoord.txt";

    //Type of Transfer function    (-t loc)    [string]    
    args.push("-t");
    args.push(parameters.tftype);

    //z_axis
    args.push("-z");
    args.push("phase");

    //input file    
    args.push(curPath + "/file/" + parameters.zippath);

    //output file   
    args.push(curPath + "/file/" + filename);


    console.log("args=", args);
    runner('harktoolcli-zip2coord', args, filename, res);
};

/////////////////////////////////////////////
//// genzip2IFFTcoord
////
exports.genzip2IFFTcoord = function(req, res) {
    var parameters = req.body;
    var cmd = 'harktoolcli-zip2coord ';
    var curPath = process.cwd();

    console.log("exports.genzip2IFFTcoord start ");
    console.log(parameters);

    var args = [];

    // var filename = "genzip2AMPcoord.txt";
    var filename = "genzip2IFFTcoord.txt";

    //Type of Transfer function    (-t loc)    [string]    
    args.push("-t");
    args.push(parameters.tftype);

    //ifft
    args.push("-i");

    //input file    
    args.push(curPath + "/file/" + parameters.zippath);

    //output file   
    args.push(curPath + "/file/" + filename);

    console.log('harktoolcli-zip2coord' + args.join(" "));
    runner('harktoolcli-zip2coord', args, filename, res);
};

exports.progress = function(req, res) {
    res.status(200).send("" + progress);
};

exports.utilpath = function(req, res) {
    which('harktool5', function(er, resolvedPath) {
        if (er) {
            res.status(200).send("");
        } else {
            var utilpath = path.dirname(resolvedPath);
            utilpath = path.join(utilpath, "..", "share", "hark") + path.sep;
            res.status(200).send(utilpath);
        }
    });
};
